"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cspell_util_bundle_1 = require("cspell-util-bundle");
const gensequence_1 = require("gensequence");
const Text = require("./text");
const path = require("path");
const fs_extra_1 = require("fs-extra");
const Trie = require("cspell-trie-lib");
const iterateWordsFromFile_1 = require("./iterateWordsFromFile");
const fileWriter_1 = require("./fileWriter");
const util_1 = require("hunspell-reader/dist/util");
const regNonWordOrSpace = cspell_util_bundle_1.xregexp("[^\\p{L}' ]+", 'gi');
const regExpSpaceOrDash = /(?:\s+)|(?:-+)/g;
const regExpRepeatChars = /(.)\1{3,}/i;
function normalizeWords(lines) {
    return lines.concatMap(line => lineToWords(line));
}
exports.normalizeWords = normalizeWords;
function lineToWords(line) {
    // Remove punctuation and non-letters.
    const filteredLine = line.replace(regNonWordOrSpace, '|');
    const wordGroups = filteredLine.split('|');
    const words = gensequence_1.genSequence(wordGroups)
        .concatMap(a => [a, ...a.split(regExpSpaceOrDash)])
        .concatMap(a => splitCamelCase(a))
        .map(a => a.trim())
        .filter(a => !!a)
        .filter(s => !regExpRepeatChars.test(s))
        .map(a => a.toLowerCase());
    return words;
}
exports.lineToWords = lineToWords;
function splitCamelCase(word) {
    const splitWords = Text.splitCamelCaseWord(word);
    // We only want to preserve this: "New York" and not "Namespace DNSLookup"
    if (splitWords.length > 1 && regExpSpaceOrDash.test(word)) {
        return gensequence_1.genSequence(splitWords).concatMap(w => w.split(regExpSpaceOrDash));
    }
    return splitWords;
}
async function compileWordList(filename, destFilename, options) {
    const pWords = iterateWordsFromFile_1.streamWordsFromFile(filename, options);
    const destDir = path.dirname(destFilename);
    const pDir = fs_extra_1.mkdirp(destDir);
    const compile = options.splitWords ? compileWordListWithSplitSeq : compileSimpleWordListSeq;
    const words = await pWords;
    const seq = compile(words)
        .filter(a => !!a)
        .filter(util_1.uniqueFilter(10000));
    const finalSeq = options.sort ? gensequence_1.genSequence(sort(seq)) : seq;
    await pDir;
    return fileWriter_1.writeSeqToFile(finalSeq.map(a => a + '\n'), destFilename);
}
exports.compileWordList = compileWordList;
function compileWordListWithSplit(filename, destFilename, options = { splitWords: true, sort: true }) {
    const { sort = true } = options;
    return compileWordList(filename, destFilename, Object.assign(Object.assign({}, options), { splitWords: true, sort }));
}
exports.compileWordListWithSplit = compileWordListWithSplit;
async function compileSimpleWordList(filename, destFilename, options = { splitWords: false, sort: true }) {
    const { sort = true } = options;
    return compileWordList(filename, destFilename, Object.assign(Object.assign({}, options), { splitWords: false, sort }));
}
exports.compileSimpleWordList = compileSimpleWordList;
function sort(words) {
    return [...words].sort();
}
function compileWordListWithSplitSeq(words) {
    return words.concatMap(line => lineToWords(line).toArray());
}
function compileSimpleWordListSeq(words) {
    return words.map(a => a.toLowerCase());
}
function normalizeWordsToTrie(words) {
    const result = normalizeWords(words)
        .reduce((node, word) => Trie.insert(word, node), {});
    return result;
}
exports.normalizeWordsToTrie = normalizeWordsToTrie;
async function compileWordListToTrieFile(words, destFilename) {
    const destDir = path.dirname(destFilename);
    const pDir = fs_extra_1.mkdirp(destDir);
    const pRoot = normalizeWordsToTrie(words);
    const [root] = await Promise.all([pRoot, pDir]);
    return fileWriter_1.writeSeqToFile(Trie.serializeTrie(root, { base: 32, comment: 'Built by cspell-tools.' }), destFilename);
}
exports.compileWordListToTrieFile = compileWordListToTrieFile;
async function compileTrie(filename, destFilename, options) {
    const words = await iterateWordsFromFile_1.streamWordsFromFile(filename, options || {});
    return compileWordListToTrieFile(words, destFilename);
}
exports.compileTrie = compileTrie;
//# sourceMappingURL=wordListCompiler.js.map